package edu.uoc.framework.action;

import java.util.Map;

import edu.uoc.framework.helpers.Data;

/**
 * <p>Clase que encapsula el resultado tras la ejecución de un {@link Action}. Éste resultado
 *  recoge los datos y el recurso al que hay que entregar estos datos.</p>
 * 
 * <p>Las clases que implementan {@link Action} como valor de retorno tras la ejecución de su 
 * método execute(), utilizarán esta clase para indicar a qué recurso entregar los datos.</p>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ActionResponse extends Data
{
	/**
	 * Recurso destino.
	 */
	private String resource; 
	
	/**
	 * Método que contruye una ActionResponse a partir de un diccionario
	 * de pares <Clave,Valor> que contendrán los datos a enviar, y del 
	 * recurso al que deberán enviarse. 
	 */
	public ActionResponse(Map<String, Object> data, String resource) {
		super (data);
		this.resource = resource;
	}
	
	/**
	 * Método que contruye una ActionResponse en la que no hay datos
	 * que entregar al destino indicado en 'resource'.
	 */
	public ActionResponse(String resource) {
		super(null);
		this.resource = resource;
	}
	
	/**
	 * Método que construye una ActionResponse formada por un único par <Clave,Valor> 
	 * que se enviará al destino indicado en 'resource'.
	 */
	public ActionResponse(String key, Object value, String resource) {
		super (key, value);
		this.resource = resource;
	}

	/**
	 * Método que permite consultar el recurso destino.
	 */
	public String getResource() {
		return resource;
	}
}