package edu.uoc.framework.core;

import java.util.ResourceBundle;

import edu.uoc.framework.utilities.StringUtils;

/**
 * <p>Clase que implementa el patrón singleton y 
 * provee al framework de una clase que proporciona
 * métodos para la recuperación de mensajes ubicados
 * en un fichero de recursos a partir de la clave que
 * los identifica.</p>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class MessageResourceBundle {

	private static MessageResourceBundle instance;

	private static ResourceBundle resourceBundle = null;

	private MessageResourceBundle()	{
		resourceBundle = ResourceBundle.getBundle("messages");
	}

	private synchronized static void createInstance()
	{
        if (instance == null)
        	instance = new MessageResourceBundle();
    }

	/**
	 * Método que recupera un mensaje de un fichero de recursos
	 * a partir de su clave.
	 * 
	 * @param key Clave que identifica el mensaje a cargar del fichero de recursos.
	 * @return mensaje asociado a la clave key si se ha encontrado, en caso contrario 
	 * un mensaje que muestra la clave que falta por definir en el fichero de recursos.
	 */
    public static String getMessage(String key)
    {
    	String message = null;
    	createInstance();
		if (resourceBundle.containsKey(key))
			message = resourceBundle.getString(key);
		else
			message = " ¡¡¡'" + key + "'!!!";	

		return message;
    }
    
    /**
     * <p>Método que recupera un mensaje del fichero de recursos, a partir de la clave
     * que lo identifica y sustituyendo las marcas {i} dónde i es un número natural
     * superior a 1 que representa el valor del i-ésimo parámetro.</p>
     * 
     * @param key Clave que identifica el mensaje a cargar del fichero de recursos.
     * @param values Valores que quieren incrustarse en las marcas {i} del mensaje a cargar.
     * @return Mensaje perteneciente a la clave key con sus marcas {i} sustituidas por los valores
     * pasados en la llamada. En caso de no encontrarse la clave key en el fichero de recursos
     * se devolverá un mensaje indicando el nombre de la clave que falta por definir en este
     * fichero.
     */
    public static String getMessage(String key, Object... values) {
		return StringUtils.format(getMessage(key), values);
    }

    @Override
	public Object clone()
    throws CloneNotSupportedException {
		throw new CloneNotSupportedException();
	}
}
