package edu.uoc.framework.core.configuration.adapters;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.bind.annotation.adapters.XmlAdapter;

import edu.uoc.framework.core.configuration.Exceptions;
import edu.uoc.framework.core.configuration.Exception;

/**
 *<p>Clase adaptador utilizada por JAXB para definir cómo debe realizarse el proceso de marshal y unmarshal de
 *las {@link edu.uoc.framework.core.configuration.Exceptions} recuperadas del fichero de configuración del framework.</p>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ExceptionsMapAdapter extends XmlAdapter<Exceptions, Map<String, Exception>> {

	@Override
	public Exceptions marshal(Map<String, Exception> v) throws java.lang.Exception {
		return null;
	}

	/**
	 * Método que durante el proceso de unmarshal del objeto {@link edu.uoc.framework.core.configuration.Exceptions} 
	 * (que contiene una colección de clases {@link edu.uoc.framework.core.configuration.Exception})
	 * convierte éste en una tabla de hash en donde las claves son tipo de la excepción
	 * y el valor el objeto {@link edu.uoc.framework.core.configuration.Exception}.</p>
	 */
	@Override
	public Map<String, Exception> unmarshal(Exceptions exceptions) 
	throws java.lang.Exception 
	{
		Map<String, Exception> excepciones = new HashMap<String,Exception>();
		Exception currentException = null;
		for (Iterator<Exception> iterator = exceptions.getException().iterator(); iterator.hasNext();) 
		{
			currentException = iterator.next();
			excepciones.put(currentException.getType(), currentException);
		}
		return excepciones;
	} 
}
