package edu.uoc.framework.exception;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.helpers.Data;

/**
 * <p>Interfaz que define el contrato necesario a implementar
 * si se desea que el framework ejecute cierta lógica ante una situación de error.</p>
 * 
 * <p>En el fichero de configuración del framework se declarará la excepción a atrapar 
 * junto con la clase que se ejecutará como resultado.</p>  
 * 
 * <pre>
 *{@code
 *<jmvc:exceptions>
 *	<jmvc:exception jsp="/jsp/error.jsp" type="java.lang.Exception" handler="edu.uoc...."/>
 *</jmvc:exceptions>
 *}
 *</pre>
 *
 * <ul><li>La propiedad 'handler' representará la clase Java que implementará esta interfaz y que llevará
 * a cabo, en su método execute, la lógica asociado con el tratamiento de dicha excepción.</li> 
 * 
 * <li>La propiedad 'jsp' indica la vista que se mostrará una vez ejecutada la lógica asociada con el 
 * tratamiento de la excepción.</li>
 * 
 * <li>La propiedad 'type' indica la excepción que desea atraparse.</li></ul>
 *  
 * @author Alejandro Marmelo Insua
 *
 */
public interface ExceptionHandler {

	/**
	 * Método que implementará la lógica a ejecutarse ante una situación de error.
	 * 
	 * @param contextObject Objeto que contiene los datos del contexto vinculados con la petición que se 
	 * 						está sirviendo.
	 * @param form ActionForm que contiene los datos validados antes de ejecutar un Action. 
	 * 				Este campo podrá o no contener datos en función del momento el que se haya
	 * 				producido la situación de error y de si el Action en ejecución tiene o no 
	 * 				ActionForm asociado. 
	 * 
	 * @param exception Excepción lanzada como resultado de la situación de error.
	 * @return Objeto Data que contendrá la información que se enviará a la vista.
	 * @throws Exception Excepción que indicará una condición de error durante la ejecución del manejador de excepción. 
	 */
	public Data execute(ContextObject contextObject, Form form, Exception exception) throws Exception; 
}
