package edu.uoc.framework.form;

import edu.uoc.framework.core.ContextObject;

/**
 * 
 * <p>Interfaz que define el contrato que una clase deberá seguir
 * si ésta desea poder capturar y validar los parámetros recibidos
 * en una petición.</p>
 * 
 * <p>Toda clase que implemente esta interfaz deberá definir tantas propiedades
 * como parámetros reciba y quiera capturar del formulario recibido.</p>
 * <p>Los nombres de estas propiedades de la clase deben coincidir con el 
 * campo name de los componentes HTML utilizados para enviar los datos.</p>
 * 
 * Ejemplo:
 *  
 * Si en un formulario HTML tenemos el componente: 
 * 
 * {@code <input type="text" name="nombre">}
 *  
 *   <p>Cuando el formulario se envie al servidor, la implementación de Form
 *   deberá tener un atributo llamado 'nombre' y disponer de sus métodos
 *   get y set según la convención JavaBean.</p>
 *
 *   <pre>
 *   private String nombre;
 *   
 *   public String getNombre() {
 *   	return nombre;
 *   }
 *   
 *   public void setNombre(String nombre) {
 *      this.nombre = nombre;
 *   }
 *   </pre>
 * 
 * <p>Habitualmente las clases que implementan esta interfaz
 * desean poder capturar los valores enviados en un formulario HTML,
 * por lo que se define la firma del método validate, mediante el cual 
 * se permitirá llevar a cabo validaciones simples sobre los parámetros
 * de entrada recibidos, permitiendo devolver los datos recibidos junto con
 * los errores encontrados sin llegar a ejecutar el {@link edu.uoc.framework.action.Action} vinculado con
 * la petición recibida.</p>
 * 
 * <p>Ejemplo de declaración de un Form en el fichero de configuración.</p>
 * <pre>
 *{@code
 *<jmvc:forms>
 *	<jmvc:form name="LoginForm" class="edu.uoc.presentation.forms.LoginForm"/>
 *</jmvc:forms>
 * *}
 *</pre>
 *  
 * <p>Elementos de un Form</p>
 *  <ul><li>Name - Atributo obligatorio que indica el nombre del Form.</li>
 * <li>Class - Atributo obligatorio que indica la clase que implementará Form.</li>
 *
 * @author Alejandro Marmelo Insua
 *
 */
public interface Form {

	/**
	 * Método desde el que se permite la validación de los datos recibidos 
	 * del cliente. 
	 * 
	 * @param contextObject Objeto que encapsula los datos de contexto de la petición recibida.
	 * @return Objeto {@link FormErrors} con contenido si hay errores que devolver al
	 * propietario del formulario recibido. null u objeto sin errores si las 
	 * validaciones han sido satisfactorias.
	 * @throws Exception Representa una situación de error sucedida durante la ejecución del método.
	 */
	public FormErrors validate(ContextObject contextObject) throws Exception;
}
