package edu.uoc.framework.helpers;

import java.util.Map;

import edu.uoc.framework.helpers.Data;

/**
 * <p>Clase interna de soporte que el framework utiliza para 
 * encapsular los datos a enviar y el recurso al que van 
 * destinados sabiendo si el envío a éste debe hacerse 
 * mediante una redirección o no.</p> 
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ResourceAndData extends Data
{
	/**
	 * Propiedad que guarda el nombre del recurso destino.
	 */
	private String resource;
	/**
	 * Propiedad que indica si el envio de datos al recurso destino se hará por redirección o no.
	 */
	private boolean redirect;
	
	public ResourceAndData(Map<String,Object> data, String resource, boolean redirect) {
		super(data);
		this.resource = resource;
		this.redirect = redirect;
	}
	
	public ResourceAndData(String key, Object value, String resource, boolean redirect) {
		super(key, value);
		this.resource = resource;
		this.redirect = redirect;
	}
	
	/**
	 * Método que recupera el valor de la propiedad resource.
	 * @return valor de la propiedad resource.
	 */
	public String getResource() {
		return resource;
	}

	/**
	 * Método que consulta el valor de la propiedad redirect.
	 * @return valor de la propiedad redirect.
	 */
	public boolean isRedirect() {
		return redirect;
	}
}