package edu.uoc.framework.utilities.I18N;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Date;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

import edu.uoc.framework.core.MessageResourceBundle;
import edu.uoc.framework.exception.InternationalizationException;

/**
 * Clase que proporciona una colección de métodos de ayuda en la internalización
 * de fechas, horas, números y monedas.
 * 
 * @author Alejandro Marmelo Insua
 */
public class I18n 
{
	public static final String I18N_CONFIGURATION_MISSING = "i18n.configuration.missing";
	public static final String I18N_KEY_MISSING = "i18n.key.missing";
	private static Logger log = Logger.getLogger(I18n.class);
		
	/**
	 * Enumeración que indica los formatos de fechas permitidos.
	 */
	public static enum DATE_TIME_FORMAT {Default, Short, Medium, Long, Full};
	
	/**
	 * Enumeración que indica los tipos de conversiones permitidas.
	 */
	public static enum I18N_TYPES {Currency, Number, Decimal, Date, Time, Date_Time};
	
	/**
	 * Método que dada la clave que identifica el mensaje, el nombre base de un paquete de recursos
	 * y un locale, recuperar el mensaje asociado a esa clave en el fichero de recursos asociado al 
	 * locale.
	 *  
	 * @param baseName nombre del paquete de recursos.
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param key clave que identifica un mensaje concreto del paquete de recursos.
	 * @return String asociado a la clave 'key' del paquete de recursos 'baseName' asociado a la región indicada por 'locale'.
	 * @throws InternationalizationException Excepción que indica un error sucedido durante la recuperación del mensaje.
	 */
	public static String getText(String baseName, Locale locale, String key)
	throws InternationalizationException 
	{
		ResourceBundle resourceBundle = null;
		String text = null;
		try
		{		
			resourceBundle = ResourceBundle.getBundle(baseName, locale);
			if (resourceBundle!=null)
				text = resourceBundle.getString(key);
		}
		catch (NullPointerException e)
		{ //Si baseName o Locale son null.
			throw new InternationalizationException(
					MessageResourceBundle.getMessage(I18N_CONFIGURATION_MISSING, baseName, locale.getDisplayName()));                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               
		}
		catch (MissingResourceException e1)
		{//Se produce cuando la clave que se intenta recuperar no existe en el bundle.
			log.warn(MessageResourceBundle.getMessage(I18N_KEY_MISSING,key,baseName));
			text = "¡¡¡" + key + "!!!";
		}
		return text;
	}
	
	/**
	 * Método que permite formatear un número de tipo long según la región indicada en 'locale'. 
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param number número que desea convertirse.
	 * @return String que contiene la representación de 'number' según la región indicada 'locale'.
	 */
	public static String getFormattedNumber(Locale locale, long number) {
		return NumberFormat.getInstance(locale).format(number);
	}
	
	
	/**
	 * Método que permite formatear un número de tipo double según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param number número que desea formatearse.
	 * @return String que contiene la representación de 'number' según la región indicada en 'locale'.
	 */
	public static String getFormattedNumber(Locale locale, double number) {
		return NumberFormat.getInstance(locale).format(number);
	}
	
	/**
	 * Método que permite formatear un objeto como un número según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param number objeto que contiene el número que desea formatearse.
	 * @return String que contiene la representación de 'number' según la región indicada en 'locale'.
	 */
	public static String getFormattedNumber(Locale locale, Object number) {
		return NumberFormat.getInstance(locale).format(number);
	}
	
	/**
	 * Método que permite formatear una moneda según la región indicada en 'locale'.
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param currency moneda que desea formatearse.
	 * @return String que contiene la representación de 'currency' según la región indicada en 'locale'.
	 */
	public static String getFormattedCurrency(Locale locale, double currency) {
		return NumberFormat.getCurrencyInstance(locale).format(currency);
	}
	
	/**
	 * Método que permite formatear una moneda según la región indicada en 'locale'.
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param currency moneda que desea formatearse.
	 * @return String que contiene la representación de 'currency' según la región indicada en 'locale'.
	 */
	public static String getFormattedCurrency(Locale locale, long currency) {
		return NumberFormat.getCurrencyInstance(locale).format(currency);
	}
	
	/**
	 * Método que permite formatear una moneda según la región indicada en 'locale'.
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param currency moneda que desea formatearse.
	 * @return String que contiene la representación de 'currency' según la región indicada en 'locale'.
	 */
	public static String getFormattedCurrency(Locale locale, Object currency) {
		return NumberFormat.getCurrencyInstance(locale).format(currency);
	}
	
	/**
	 * Método que permite formatear un número decimal según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param decimal número decimal que desea formatearse.
	 * @return String que contiene la representación de 'decimal' según la región indicada en 'locale'.
	 */
	public static String getFormattedDecimal(Locale locale, long decimal) {
		return ((DecimalFormat) NumberFormat.getNumberInstance(locale)).format(decimal);
	}
	
	/**
	 * Método que permite formatear un número decimal según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param decimal número decimal que desea formatearse.
	 * @return String que contiene la representación de 'decimal' según la región indicada en 'locale'.
	 */
	public static String getFormattedDecimal(Locale locale, double decimal) {
		return ((DecimalFormat) NumberFormat.getNumberInstance(locale)).format(decimal);
	}
	
	/**
	 * Método que permite formatear un número decimal según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param decimal número decimal que desea formatearse.
	 * @return String que contiene la representación de 'decimal' según la región indicada en 'locale'.
	 */
	public static String getFormattedDecimal(Locale locale, Object decimal) {
		return ((DecimalFormat) NumberFormat.getNumberInstance(locale)).format(decimal);
	}
	
	/**
	 * Método que permite formatear una fecha al formato indicado en 'dateFormat' según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param dateFormat formato a utilizar para representar la fecha.
	 * @param date fecha a formatear.
	 * @return String que contiene la representación de 'date' según la región indicada en 'locale' y en el formato indicado por 'dateFormat'.
	 */
	public static String getFormattedDate(Locale locale, DATE_TIME_FORMAT dateFormat, Date date) {
		return DateFormat.getDateInstance(getFormat(dateFormat), locale).format(date);
	}
	
	/**
	 * Método que permite formatear una fecha al formato indicado en 'timeFormat' según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param timeFormat formato a utilizar para representar la hora.
	 * @param date fecha a formatear.
	 * @return String que contiene la representación de 'date' según la región indicada en 'locale' y en el formato indicado por 'timeFormat'.
	 */
	public static String getFormattedTime(Locale locale, DATE_TIME_FORMAT timeFormat, Date date) {
		return DateFormat.getTimeInstance(getFormat(timeFormat), locale).format(date);
	}
	
	/**
	 * Método que permite formatear una fecha con las horas, minutos, segundo indicado en 'timeFormat' y el dia, mes,año indicado por
	 * dateFormat según la región indicada en 'locale'.
	 * 
	 * @param locale {@link java.util.Locale} que representa una región geográfica, política y cultural.
	 * @param dateFormat formato a utilizar para representar la fecha.
	 * @param timeFormat formato a utilizar para representar la hora.
	 * @param date fecha a formatear.
	 * @return String que contiene la representación de 'date' según la región indicada en 'locale' donde las horas, minutos, segundos 
	 * se han formateado según el formato indicado en 'timeFormat' y el dia, mes, año según el formato indicado por
	 * 'dateFormat'.
	 */
	public static String getFormattedDateTime(Locale locale, DATE_TIME_FORMAT dateFormat, DATE_TIME_FORMAT timeFormat, Date date) {
		return DateFormat.getDateTimeInstance(getFormat(dateFormat), getFormat(timeFormat), locale).format(date);
	}
	
	/**
	 * Método que recupera el código de formato vinculado con el formato 'format'.
	 * 
	 * @param format formato.
	 * @return código asociado al formato 'format'.
	 */
	private static int getFormat(DATE_TIME_FORMAT format)
	{
		int code = 0;
		switch (format)
		{
			case Default:
				code = DateFormat.DEFAULT;
			break;
			case Short:
				code = DateFormat.SHORT;
			break;
			case Medium:
				code = DateFormat.MEDIUM;
			break;
			case Long:
				code = DateFormat.LONG;
			break;
			case Full:
				code = DateFormat.FULL;
			break;
		}
		return code;
	}
}