package edu.uoc.framework.utilities;

/**
 * Clase que recopila un conjunto de funciones basadas en Strings
 * que son utilizadas por el framework.
 * 
 * @author Alejandro Marmelo Insua.
 *
 */
public class StringUtils {

	/**
	 * Método que dada una cadena de carácteres, sustituye los tokens de la forma {i} (donde i es un número Natural)
	 * por el elemento que ocupa esa posición en el array de valores pasado como parámetro.
	 * 
	 * @param source String al que se le desea dar formato.
	 * @param values Array de objetos que desean sustituirse en 'source'.
	 * @return Cadena en la que los tokens de la forma {i} han sido sustituidos por el
	 * i-ésimo valor del array de valores pasado cómo parámetro.
	 */
	public static String format(String source, Object[] values) 
	{
		String regExpr = null;
		if (!source.isEmpty() && values!=null)
		{
			for (int index = 1; index <= values.length; index++)
			{
				regExpr = "\\{"+index+"\\}";
				source = source.replaceFirst(regExpr,String.valueOf(values[index-1]));
			}
		}
		return source;
	}
	
	/**
	 * Método que comprueba si un String tiene contenido o no, y en caso de 
	 * tener contenido se comprueba que éste no esté formado únicamente por 
	 * espacios en blanco.
	 * 
	 * @param str String que desea comprobarse.
	 * @return true si str no tiene contenido, false en caso contrario.
	 */
	 public static boolean isBlank(String str) 
	 {
		 int strLen;
	     if (str == null || (strLen = str.length()) == 0) 
	    	 return true;
	     
	     for (int i = 0; i < strLen; i++) 
	     {
            if (!Character.isWhitespace(str.charAt(i))) 
                return false;
        }
        return true;
    }
}
