package edu.uoc.framework.view.tags;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;

import edu.uoc.framework.exception.InternationalizationException;
import edu.uoc.framework.exception.TagException;
import edu.uoc.framework.form.FormErrors;
import edu.uoc.framework.utilities.Constants;
import edu.uoc.framework.utilities.StringUtils;
import edu.uoc.framework.utilities.I18N.I18n;
import edu.uoc.framework.utilities.I18N.Message;

/**
 *
 * <p>Clase que implementa el tag Error que podrá utilizarse en una página JSP
 * y cuya funcionalidad será la de mostrar mensajes de error.</p>
 * 
 *  Más información en el fichero "JavaMVC.i18n.tld" 
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ErrorTag implements Tag
{
	private PageContext pageContext;
	private Tag parent;
	
	private String bundle;
	private String locale;
	private String property;
	private String name;
	private String header = "";
	private String prefix = "";
	private String suffix = "";
	private String footer = "";
	
	/**
	 * Método que recupera el valor de la propiedad 'pageContext'.
	 * @return {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public PageContext getPageContext() {
		return pageContext;
	}

	/**
	 * Método que establece el valor de la propiedad 'pageContext'.
	 * @param pageContext {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public void setPageContext(PageContext pageContext) {
		this.pageContext = pageContext;
	}

	/**
	 * Método que recupera el valor de la propiedad 'parent'.
	 * @return {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public Tag getParent() {
		return parent;
	}

	/**
	 * Método que establece el valor de la propiedad 'parent'.
	 * @param parent {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public void setParent(Tag parent) {
		this.parent = parent;
	}

	/**
	 * Método que recupera el valor de la propiedad 'bundle'.
	 * @return {@link String} con el valor de la propiedad 'bundle'.
	 */
	public String getBundle() {
		return bundle;
	}

	/**
	  * Método que establece el valor de la propiedad 'bundle'.
	 * @param bundle {@link String} con el valor de la propiedad 'bundle'.
	 */
	public void setBundle(String bundle) {
		this.bundle = bundle;
	}

	/**
	 * Método que recupera el valor de la propiedad 'locale'.
	 * @return {@link String} con el valor de la propiedad 'locale'.
	 */
	public String getLocale() {
		return locale;
	}

	/**
	 * Método que establece el valor de la propiedad 'locale'.
	 * @param locale {@link String} con el valor de la propiedad 'locale'.
	 */
	public void setLocale(String locale) {
		this.locale = locale;
	}

	/**
	  * Método que recupera el valor de la propiedad 'property'.
	 * @return {@link String} con el valor de la propiedad 'property'.
	 */
	public String getProperty() {
		return property;
	}

	/**
	 * Método que establece el valor de la propiedad 'property'.
	 * @param property {@link String} con el valor de la propiedad 'property'.
	 */
	public void setProperty(String property) {
		this.property = property;
	}

	/**
	  Método que recupera el valor de la propiedad 'name'.
	 * 
	 * @return {@link String} con el valor de la propiedad 'name'.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Método que establece el valor de la propiedad 'name'.
	 * @param name {@link String} con el valor de la propiedad 'name'.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Método que recupera el valor de la propiedad 'header'.
	 * @return {@link String} con el valor de la propiedad 'header'.
	 */
	public String getHeader() {
		return header;
	}

	/**
	 * Método que establece el valor de la propiedad 'header'.
	 * @param header {@link String} con el valor de la propiedad 'header'.
	 */
	public void setHeader(String header) {
		this.header = header;
	}

	/**
	 * Método que recupera el valor de la propiedad 'prefix'.
	 * @return {@link String} con el valor de la propiedad 'prefix'.
	 */
	public String getPrefix() {
		return prefix;
	}

	/**
	 * Método que establece el valor de la propiedad 'prefix'.
	 * @param prefix {@link String} con el valor de la propiedad 'prefix'.
	 */
	public void setPrefix(String prefix) {
		this.prefix = prefix;
	}

	/**
	 * Método que recupera el valor de la propiedad 'suffix'.
	 * @return {@link String} con el valor de la propiedad 'suffix'.
	 */
	public String getSuffix() {
		return suffix;
	}

	/**
	 * Método que establece el valor de la propiedad 'suffix'.
	 * @param suffix {@link String} con el valor de la propiedad 'suffix'.
	 */
	public void setSuffix(String suffix) {
		this.suffix = suffix;
	}

	/**
	 * Método que recupera el valor de la propiedad 'footer'.
	 * @return {@link String} con el valor de la propiedad 'footer'.
	 */
	public String getFooter() {
		return footer;
	}

	/**
	 * Método que establece el valor de la propiedad 'footer'.
	 * @param footer {@link String} con el valor de la propiedad 'footer'.
	 */
	public void setFooter(String footer) {
		this.footer = footer;
	}

	/**
	 * Método que procesa el tag de inicio y es invocado por el objeto que implementa la página JSP.
	 */
	public int doStartTag() 
	throws JspException 
	{
		String message = null;
		JspWriter out = null;
		try
		{
			out = pageContext.getOut();
			message = formatMessages();
			if (!StringUtils.isBlank(message))
				out.write(message);
		}
		catch (TagException ie)
		{
			throw new JspException(ie);
		}
		catch (IOException ioe)
		{
			throw new JspException(ioe);
		}
		//Si el tag contiene contenido éste se ignora.
		return Tag.SKIP_BODY;
	}
	
	/**
	 * Método que procesa el tag de fin y es invocado por el objeto que implementa la página JSP en todos los handlers de etiquetas.
	 */
	public int doEndTag()
	throws JspException {
		return Tag.EVAL_PAGE;
	}
	
	private String formatMessages()
	throws TagException
	{
		StringBuffer messages = new StringBuffer(100);
		List<String> errorMessages = getErrors();
		if (errorMessages!=null && errorMessages.size()>0)
		{
			messages.append(getHeader());
			for (String currentError : errorMessages)
			{
				messages.append(getPrefix());
				messages.append(currentError);
				messages.append(getSuffix());
			}
			messages.append(getFooter());
		}
		return messages.toString();
	}
	
	/**
	 * Método encargado de recuperar el listado de mensajes de error para mostrarlos en la JSP. 
	 */
	private List<String> getErrors()
	throws TagException
	{
		Locale locale = null;
		String bundle = null;
		String message = null;
		Map<String, Message> errors = null;
		List<String> recoveredErrors = null;
		Message currentMessage = null;
		
		//Se recupera el locale a utilizar
		locale = TagUtils.recoverLocale(getLocale(), pageContext);
		
		//Se recupera el bundle a utilizar.
		bundle = TagUtils.recoverBundle(getBundle(), pageContext);
		
		//Se recupera el Map con los errores.
		errors = recoverMap();
		if (errors!=null && errors.size()>0)
		{ //Si hay errores a tratar
			try
			{	//Si la propiedad 'property' NO se ha informado implica que se quiere recuperar
				//todo el listado de errores contenidos en la colección
				if (StringUtils.isBlank(getProperty()))
				{
					recoveredErrors = new ArrayList<String>(errors.size());
					for (String currentKey : errors.keySet())
					{
						currentMessage = errors.get(currentKey);
						if (currentMessage!=null && currentMessage.isLocalized())
						{
							message = I18n.getText(bundle, locale, currentMessage.getKey());
							message = StringUtils.format(message, currentMessage.getValues());
							recoveredErrors.add(message);
						}
						else
							recoveredErrors.add(currentMessage.getKey());
					}
				}
				else
				{	//Se quiere recuperar un error en concreto de la colección de errores.
					recoveredErrors = new ArrayList<String>(1);
					currentMessage = errors.get(getProperty());
					if (currentMessage!=null)
					{
						if (currentMessage.isLocalized())
						{
							message = I18n.getText(bundle, locale, currentMessage.getKey());
							message = StringUtils.format(message, currentMessage.getValues());
						}
						else
							message = currentMessage.getKey();
						
						recoveredErrors.add(message);
					}
				}
			}
			catch (InternationalizationException ie)
			{
				throw new TagException(ie);
			}
		}
		return recoveredErrors;
	}
	
	/**
	 * Método que recupera de la request un Map<String, Message> con los mensajes de error
	 * a mostrar. El tag permite especificar el nombre del atributo que contiene dichos mensajes. 
	 * En ausencia de este campo se recuperarán los mensajes de error incluidos en la request y vinculados 
	 * al atributo definido por la constante del framework 'Constants.JavaMVC_ErrorMessages'.
	 * @return Diccionario de pares <Clave, Message> con los errores recuperados de la request
	 */
	@SuppressWarnings("unchecked")
	private Map<String, Message> recoverMap()
	{
		String name = getName();
		Map<String, Message> errorMap = null;
		if (StringUtils.isBlank(name))
		{
			FormErrors formErrors = (FormErrors)pageContext.getRequest().getAttribute(Constants.JavaMVC_ErrorMessages);
			if (formErrors!=null)
				errorMap = formErrors.getErrors();
		}
		else
			errorMap = (Map<String, Message>) pageContext.getRequest().getAttribute(getName());
		
		return errorMap;
	}

	/**
	 * Método que libera los recursos utilizados por el Tag.
	 */
	public void release(){
		bundle = null;
		locale = null;
		property = null;
		name = null;
		header = null;
		prefix = null;
		suffix = null;
		footer = null;
	}
}