package edu.uoc.framework.view.tags;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;

import edu.uoc.framework.utilities.Constants;
import edu.uoc.framework.utilities.StringUtils;

/**
*
* Clase que implementa el tag Form que podrá utilizarse en una página JSP
* y cuya funcionalidad es la de generar un formulario HTML.
* 
*  Más información en el fichero "JavaMVC.html.tld" 
* 
* @author Alejandro Marmelo Insua
*
*/
public class FormTag implements Tag
{
	private PageContext pageContext;
	private Tag parent;
	
	private String id;
	private String action;
	private String method;
	private String enctype;
	private String target;
	private String styleClass;
	private String style;
	
	/**
	 * Método que recupera el valor de la propiedad 'pageContext'.
	 * @return {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public PageContext getPageContext() {
		return pageContext;
	}

	/**
	 * Método que establece el valor de la propiedad 'pageContext'.
	 * @param pageContext {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public void setPageContext(PageContext pageContext) {
		this.pageContext = pageContext;
	}

	/**
	 * Método que recupera el valor de la propiedad 'parent'.
	 * @return {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public Tag getParent() {
		return parent;
	}

	/**
	 * Método que establece el valor de la propiedad 'parent'.
	 * @param parent {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public void setParent(Tag parent) {
		this.parent = parent;
	}

	/**
	 * Método que recupera el valor de la propiedad 'action'.
	 * @return {@link String} con el valor de la propiedad 'action'.
	 */
	public String getAction() {
		return action;
	}

	/**
	 * Método que establece el valor de la propiedad 'action'.
	 * @param action {@link String} con el valor de la propiedad 'action'.
	 */
	public void setAction(String action) {
		this.action = action;
	}

	/**
	 * Método que recupera el valor de la propiedad 'method'.
	 * @return {@link String} con el valor de la propiedad 'method'.
	 */
	public String getMethod() {
		return method;
	}

	/**
	 * Método que establece el valor de la propiedad 'method'.
	 * @param method {@link String} con el valor de la propiedad 'method'.
	 */
	public void setMethod(String method) {
		this.method = method;
	}

	/**
	 * Método que recupera el valor de la propiedad 'enctype'.
	 * @return {@link String} con el valor de la propiedad 'enctype'.
	 */
	public String getEnctype() {
		return enctype;
	}

	/**
	 * Método que establece el valor de la propiedad 'enctype'.
	 * @param enctype {@link String} con el valor de la propiedad 'enctype'.
	 */
	public void setEnctype(String enctype) {
		this.enctype = enctype;
	}

	/**
	 * Método que recupera el valor de la propiedad 'target'.
	 * @return {@link String} con el valor de la propiedad 'target'.
	 */
	public String getTarget() {
		return target;
	}

	/**
	 * Método que establece el valor de la propiedad 'target'.
	 * @param target {@link String} con el valor de la propiedad 'target'.
	 */
	public void setTarget(String target) {
		this.target = target;
	}

	/**
	 * Método que recupera el valor de la propiedad 'styleClass'.
	 * @return {@link String} con el valor de la propiedad 'styleClass'.
	 */
	public String getStyleClass() {
		return styleClass;
	}

	/**
	 * Método que establece el valor de la propiedad 'styleClass'.
	 * @param styleClass {@link String} con el valor de la propiedad 'styleClass'.
	 */
	public void setStyleClass(String styleClass) {
		this.styleClass = styleClass;
	}

	/**
	 * Método que recupera el valor de la propiedad 'style'.
	 * @return {@link String} con el valor de la propiedad 'style'.
	 */
	public String getStyle() {
		return style;
	}

	/**
	 * Método que establece el valor de la propiedad 'style'.
	 * @param style {@link String} con el valor de la propiedad 'style'.
	 */
	public void setStyle(String style) {
		this.style = style;
	}

	/**
	 * Método que recupera el valor de la propiedad 'id'.
	 * @return {@link String} con el valor de la propiedad 'id'.
	 */
	public String getId() {
		return id;
	}

	/**
	 * Método que establece el valor de la propiedad 'id'.
	 * @param id {@link String} con el valor de la propiedad 'id'.
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * Método que procesa el tag de inicio y es invocado por el objeto que implementa la página JSP.
	 */
	public int doStartTag() 
	throws JspException 
	{
		JspWriter out = null;
		StringBuffer results = new StringBuffer("<form");
		try
		{
			renderAttribute(results, "method", (getMethod() == null) ? "post" : getMethod());
			renderAttribute(results, "target", getTarget());
			renderAttribute(results, "enctype", getEnctype());
			renderAttribute(results, "class", getStyleClass());
			renderAttribute(results, "style", getStyle());
			renderAttribute(results, "id", getId());
			renderActionAttribute(results, "action", getAction());
			results.append(">");
			
			out = pageContext.getOut();
			out.write(results.toString());
		}
		catch (IOException ioe)
		{
			throw new JspException(ioe);
		}
		//Si el tag contiene contenido éste se ignora.
		return Tag.EVAL_BODY_INCLUDE;
	}
	
	/**
	 * Método que procesa el tag de fin y es invocado por el objeto que implementa la página JSP en todos los handlers de etiquetas.
	 */
	public int doEndTag()
	throws JspException
	{
		JspWriter out = null;
		try
		{
			out = pageContext.getOut();
			out.write("</form>");
		}
		catch (IOException ioe)
		{
			throw new JspException(ioe);
		}
		return Tag.EVAL_PAGE;
	}

	private void renderActionAttribute(StringBuffer results, String attribute, String value)
	{
		results.append(" action=\"");
		results.append(formatAction());
        results.append("\"");
	}
	
	/**
	 * Método encargado de construir la ruta a la que se enviará el formulario  
	 * una vez que se haga el submit de éste.
	 * 
	 * El método construirá la ruta teniendo en cuenta el patrón de urls que serán
	 * procesadas por el FrontController.
	 */
	private String formatAction()
	{
		String actionMapping = null;
		String queryString = null;
		StringBuffer formattedAction = new StringBuffer(10);
		String urlPattern = (String) pageContext.getServletContext().getAttribute(Constants.JavaMVC_UrlPattern);
		
		if (!StringUtils.isBlank(urlPattern))
		{
			//En primer lugar se comprueba si el contenido del atributo Action del Tag contiene una url con parámetros. 
			int question = action.indexOf("?");
			 if (question >= 0)
				 queryString = action.substring(question);
			 
			 actionMapping = getActionMappingName(action);
			 formattedAction.append(pageContext.getServletContext().getContextPath());
			 
			 //Mapeo por extensión - Sigue el patrón *.xxx NUNCA empezará con /
			 if (urlPattern.startsWith("*.")) { 
				 formattedAction.append(actionMapping);
				 formattedAction.append(urlPattern.substring(1));
			 }//Si nos encontramos ante un mapeo por Path - Sigue la forma /.../*
			 else if (urlPattern.endsWith("/*")) {
				 formattedAction.append(urlPattern.substring(0, urlPattern.length() - 2));
				 formattedAction.append(actionMapping);
			 }//Servlet por defecto.
			 else if (urlPattern.equals("/")) 
				 formattedAction.append(actionMapping);
			 else //Caso sólo para mapeos exactos
				 formattedAction.append(urlPattern);
			 
			 //Si el action contenido QueryString se le añade nuevamente al final.
			 if (queryString!=null)
				 formattedAction.append(queryString);
		}
		else
		{ //En el caso en el que no hubiese podido recuperar el URL-PATTERN para el FrontController
			//habrá que asegurarse de que cualquier Action empieza con el carácter /
			if (!action.startsWith("/"))
				formattedAction.append("/");
			formattedAction.append(action);
		}
		return formattedAction.toString();
	}
	
	private static String getActionMappingName(String action) 
	{
		String value = action;
		int question = action.indexOf("?");
		if (question >= 0) 
			value = value.substring(0, question);

		int slash = value.lastIndexOf("/");
		int period = value.lastIndexOf(".");
		if ((period >= 0) && (period > slash)) 
			value = value.substring(0, period);
		if (value.startsWith("/"))
			return (value);
		else
			return ("/" + value);
	}
		
	protected void renderAttribute(StringBuffer results, String attribute, String value) 
	{
		if (value != null) 
        {
			results.append(" ");
            results.append(TagUtils.filter(attribute));
            results.append("=\"");
            results.append(TagUtils.filter(value));
            results.append("\"");
        }
    }
	
	/**
	 * Método que libera los recursos utilizados por el Tag.
	 */
	public void release() {
		action=null;
		method=null;
		enctype=null;
		target=null;
		styleClass=null;
		style=null;
	}
}