package edu.uoc.framework.view.tags;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;

import edu.uoc.framework.exception.InternationalizationException;
import edu.uoc.framework.exception.TagException;
import edu.uoc.framework.utilities.StringUtils;
import edu.uoc.framework.utilities.I18N.I18n;
import edu.uoc.framework.utilities.I18N.Message;

/**
*
* Clase que implementa el tag Message que podrá utilizarse en una página JSP
* y cuya funcionalidad es la de mostrar textos a partir de las claves que los
* identifican en los ficheros de recursos de la aplicación en función del locale.
* 
*  Más información en el fichero "JavaMVC.i18n.tld" 
* 
* @author Alejandro Marmelo Insua
*
*/
public class MessageTag implements Tag
{
	private PageContext pageContext;
	private Tag parent;
	private String key;
	private String bundle;
	private String locale;
	private String attribute;
	private String scope;
	private String param1;
	private String param2;
	private String param3;
	private String param4;
	
	/**
	 * Método que recupera el valor de la propiedad 'pageContext'.
	 * @return {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public PageContext getPageContext() {
		return pageContext;
	}

	/**
	 * Método que establece el valor de la propiedad 'pageContext'.
	 * @param pageContext {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public void setPageContext(PageContext pageContext) {
		this.pageContext = pageContext;
	}

	/**
	 * Método que recupera el valor de la propiedad 'parent'.
	 *@return {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public Tag getParent() {
		return parent;
	}

	/**
	 * Método que establece el valor de la propiedad 'parent'.
	 * @param parent {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public void setParent(Tag parent) {
		this.parent = parent;
	}

	/**
	 * Método que recupera el valor de la propiedad 'key'.
	 * @return {@link String} con el valor de la propiedad 'key'.
	 */
	public String getKey() {
		return key;
	}

	/**
	 * Método que establece el valor de la propiedad 'key'.
	 * @param key {@link String} con el valor de la propiedad 'key'.
	 */
	public void setKey(String key) {
		this.key = key;
	}

	/**
	 * Método que recupera el valor de la propiedad 'key'.
	 * @return {@link String} con el valor de la propiedad 'key'.
	 */
	public String getBundle() {
		return bundle;
	}

	/**
	 * Método que establece el valor de la propiedad 'bundle'.
	 * @param bundle {@link String} con el valor de la propiedad 'bundle'.
	 */
	public void setBundle(String bundle) {
		this.bundle = bundle;
	}

	/**
	 * Método que recupera el valor de la propiedad 'key'.
	 * @return {@link String} con el valor de la propiedad 'key'.
	 */
	public String getLocale() {
		return locale;
	}

	/**
	 * Método que establece el valor de la propiedad 'locale'.
	 * @param locale {@link String} con el valor de la propiedad 'locale'.
	 */
	public void setLocale(String locale) {
		this.locale = locale;
	}

	/**
	 * Método que recupera el valor de la propiedad 'param1'.
	 * @return {@link String} con el valor de la propiedad 'param1'.
	 */
	public String getParam1() {
		return param1;
	}

	/**
	 * Método que establece el valor de la propiedad 'param1'.
	 * @param param1 {@link String} con el valor de la propiedad 'param1'.
	 */
	public void setParam1(String param1) {
		this.param1 = param1;
	}

	/**
	 * Método que recupera el valor de la propiedad 'param2'.
	 * @return {@link String} con el valor de la propiedad 'param2'.
	 */
	public String getParam2() {
		return param2;
	}

	/**
	 * Método que establece el valor de la propiedad 'param2'.
	 * @param param2 {@link String} con el valor de la propiedad 'param2'.
	 */
	public void setParam2(String param2) {
		this.param2 = param2;
	}

	/**
	 * Método que recupera el valor de la propiedad 'param3'.
	 * @return {@link String} con el valor de la propiedad 'param3'.
	 */
	public String getParam3() {
		return param3;
	}

	/**
	 * Método que establece el valor de la propiedad 'param3'.
	 * @param param3 {@link String} con el valor de la propiedad 'param3'.
	 */
	public void setParam3(String param3) {
		this.param3 = param3;
	}

	/**
	 * Método que recupera el valor de la propiedad 'param4'.
	 * @return {@link String} con el valor de la propiedad 'param4'.
	 */
	public String getParam4() {
		return param4;
	}

	/**
	 * Método que establece el valor de la propiedad 'param4'.
	 * @param param4 {@link String} con el valor de la propiedad 'param4'.
	 */
	public void setParam4(String param4) {
		this.param4 = param4;
	}

	/**
	 * Método que recupera el valor de la propiedad 'attribute'.
	 * @return {@link String} con el valor de la propiedad 'attribute'.
	 */
	public String getAttribute() {
		return attribute;
	}

	/**
	 * Método que establece el valor de la propiedad 'attribute'.
	 * @param attribute {@link String} con el valor de la propiedad 'attribute'.
	 */
	public void setAttribute(String attribute) {
		this.attribute = attribute;
	}

	/**
	 * Método que recupera el valor de la propiedad 'scope'.
	 * @return {@link String} con el valor de la propiedad 'scope'.
	 */
	public String getScope() {
		return scope;
	}

	/**
	 * Método que establece el valor de la propiedad 'scope'.
	 * @param scope {@link String} con el valor de la propiedad 'scope'.
	 */
	public void setScope(String scope) {
		this.scope = scope;
	}

	/**
	 * Método que procesa el tag de inicio y es invocado por el objeto que implementa la página JSP.
	 */
	public int doStartTag() 
	throws JspException 
	{
		String message = null;
		JspWriter out = null;
		try
		{
			out = pageContext.getOut();
			message = recoverMessage();
			if (!StringUtils.isBlank(message))
				out.write(recoverMessage());
		}
		catch (TagException ie)
		{
			throw new JspException(ie);
		}
		catch (IOException e) 
		{
			throw new JspException(e);
		}
		//Si el tag contiene contenido éste se ignora.
		return Tag.SKIP_BODY;
	}
	
	/**
	 * Método que procesa el tag de fin y es invocado por el objeto que implementa la página JSP en todos los handlers de etiquetas.
	 */
	public int doEndTag()
	throws JspException {
		return Tag.EVAL_PAGE;
	}
	
	private String recoverMessage()
	throws TagException
	{
		Object recoveredAttribute = null;
		String key = null;
		String message = null;
		Message mesg = null;
		Locale locale = TagUtils.recoverLocale(getLocale(), pageContext);
		String bundle = TagUtils.recoverBundle(getBundle(), pageContext);
		
		//Si NO se ha informado el atributo key en la etiqueta
		if ((key = getKey())==null)
		{
			recoveredAttribute = TagUtils.getAttributeFromScope(getAttribute(), getScope(), pageContext); 
			if (recoveredAttribute instanceof Message)
			{
				if ((mesg = (Message)recoveredAttribute)!=null)
				{
					//Si el mensaje es localizable, la clave a recuperar es directamente la key de message.
					if (mesg.isLocalized())
						key = mesg.getKey();
					else
					//Si el mensaje no es localizable el message es directamente la clave y no hay que internacionalizar nada.
						message = mesg.getKey();
				}
			}
			else
				//El String recuperado es directamente la clave del mensaje a mostrar.
				key = (String)recoveredAttribute;
		}
		
		//Si se ha recuperado una Key hay que internacionalizar el mensaje.
		if (!StringUtils.isBlank(key))
		{
			try
			{
				//Se recupera el mensaje en función de la configuración especificada en el Tag.
				message = I18n.getText(bundle,locale,key);
				message = StringUtils.format(message, new Object[]{param1,param2,param3,param4});
			}
			catch (InternationalizationException ie)
			{
				throw new TagException(ie);
			}
		}
		return message;
	}

	/**
	 * Método que libera los recursos utilizados por el Tag.
	 */
	public void release() {
		key = null;
		bundle = null;
		locale = null;
		attribute = null;
		scope = null;
		param1 = null;
		param2 = null;
		param3 = null;
		param4 = null;
	}
}