package edu.uoc.framework.view.tags;

import java.util.Date;
import java.util.Locale;

import javax.servlet.jsp.PageContext;

import edu.uoc.framework.utilities.Constants;
import edu.uoc.framework.utilities.StringUtils;
import edu.uoc.framework.utilities.I18N.I18n;
import edu.uoc.framework.utilities.I18N.I18n.DATE_TIME_FORMAT;
import edu.uoc.framework.utilities.I18N.I18n.I18N_TYPES;

/**
 * 
 * Clase que recoge una colección de funcionalidades
 * de soporte utilizadas por las etiquetas.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class TagUtils {

	public static String filter(String value) 
	{
		if (value == null || value.length() == 0) 
             return value;
 
         StringBuffer result = null;
         String filtered = null;
         for (int i = 0; i < value.length(); i++) 
         {
             filtered = null;
             switch (value.charAt(i)) {
                 case '<':
                	 filtered = "&lt;";
                 break;
				 case '>':
					 filtered = "&gt;";
				 break;
				 case '&':
					 filtered = "&amp;";
				 break;
				 case '"':
					 filtered = "&quot;";
				 break;
				 case '\'':
					 filtered = "&#39;";
                 break;
             }
 
             if (result == null) 
             {
                 if (filtered != null) 
                 {
                    result = new StringBuffer(value.length() + 50);
                     if (i > 0) 
                        result.append(value.substring(0, i));
                     result.append(filtered);
                 }
             }
             else 
             {
               if (filtered == null) 
                     result.append(value.charAt(i));
               else
                     result.append(filtered);
             }
       }
        return result == null ? value : result.toString();
     }
	
	
	/**
	 * Método que recupera la ruta a los ficheros de internacionalización 
	 * de la aplicación en caso de que ésta no haya sido informada 
	 */
	public static String recoverBundle(String bundle, PageContext pageContext) 
	{
		if (StringUtils.isBlank(bundle))
			bundle = (String)pageContext.getServletContext().getAttribute(Constants.JavaMVC_I18N);
		return bundle;
	}
	
	/**
	 * Método que recupera el Locale que se utilizará para internacionalizar información.
	 * Si el campo localeStr se ha informado se utilizará para recuperar la propiedad de sesión
	 * que contiene el Locale de la aplicación.
	 * En caso de no venir informado, el locale que se utilizará será el de la request. 
	 */
	public static Locale recoverLocale(String localeStr, PageContext pageContext)
	{
		Locale locale = null;
		//El locale no se ha informado explicitamente en el tag
		if (!StringUtils.isBlank(localeStr))
			locale = (Locale)pageContext.getSession().getAttribute(localeStr);
		else
		{
			//Se comprueba que en la variable de sesion Constants.JavaMVC_Locale 
			locale = (Locale)pageContext.getSession().getAttribute(Constants.JavaMVC_Locale);
			if (locale==null)
				locale = pageContext.getRequest().getLocale();
		}
		
		return locale;
	}

	/**
	 * Método que recupera el atributo 'attribute' del scope indicado en la propiedad 'scope'.
	 * En caso de no venir informado el scope por defecto será "Request".
	 */
	public static Object getAttributeFromScope(String attribute, String scope, PageContext pageContext)
	{
		Object object = null;
		if ("Session".equalsIgnoreCase(scope))
			object = pageContext.getSession().getAttribute(attribute);
		else if ("Page".equalsIgnoreCase(scope))
			object = pageContext.getAttribute(attribute);
		else if ("Application".equalsIgnoreCase(scope))
			object = pageContext.getServletContext().getAttribute(attribute);
		else
			object = pageContext.getRequest().getAttribute(attribute);
		
		return object;
	}
	
	/**
	 * Método que internacionaliza un objeto según el locale y el tipo de elemento, 
	 * pudiendo internacionalizarse monedas, números, decimales, fechas y horas.
	 * 
	 * @param value valor a internacionalizar.
	 * @param locale locale que se utilizará para la internacionalización.
	 * @param type tipo de conversión que se desea realizar pudiendo ser {Currency, Number, Decimal, Date, Time, Date_Time}
	 * @param dateFormat formato de fecha a utilizar en una conversión de tipo Date o Date_Time
	 * @param timeFormat formato de hora a utilizar en una conversión de tipo Time o Date_Time.
	 * @return Cadena internacionalizada.
	 */
	public static String internationalizeData (Object value, Locale locale, I18N_TYPES type, DATE_TIME_FORMAT dateFormat, DATE_TIME_FORMAT timeFormat)
	{
		String internationalizedValue = null;
		switch (type)
		{
			case Currency:
				internationalizedValue = I18n.getFormattedCurrency(locale, value);
			break;
			case Decimal:
				internationalizedValue = I18n.getFormattedDecimal(locale, value);
			break;
			case Number:
				internationalizedValue = I18n.getFormattedNumber(locale, value);
			break;
			case Date:
				internationalizedValue = I18n.getFormattedDate(locale, dateFormat, (Date)value);
			break;
			case Time:
				internationalizedValue = I18n.getFormattedDate(locale, dateFormat, (Date)value);
			break;
			case Date_Time:
				internationalizedValue = I18n.getFormattedDateTime(locale, dateFormat, timeFormat, (Date)value);
			break;
		}
		return internationalizedValue;
	}
}