package edu.uoc.phonebook.business;

import edu.uoc.phonebook.business.dataobject.User;


public interface IIntegrationUserController {

	/**
	 * Método que crea un usuario en el sistema.
	 * 
	 * @param user {lynk User}
	 * @return true si el usuario se ha creado en base de datos, false en cualquier otro caso.
	 */
	public boolean create(User user);

	/**
	 * Método que actualiza los datos de un usuario del sistema.
	 * @param user {lynk User}
	 * @return true si el usuario se ha actualizado en el sistema, false en cualquier otro caso.

	 */
	public boolean update(User user);
	
	/**
	 * Método que recupera los datos de un usuario a partir de su username y password
	 * 
	 * @param username Nombre de usuario
	 * @param password Clave de acceso 
	 * @return {@link user} si el usuario existe, null en caso contrario.
	 */
	public User get (String username, String password);
	
	/**
	 * Método que recupera los datos de un usuario a partir de su nombre de usuario.
	 * 
	 * @param username Nombre de usuario.
	 * @return {@link User} si el usuario existe en el sistema, null en caso contrario.
	 */
	public User getByUsername (String username);
	
	/**
	 * Método que recupera los datos de un usuario a partir de un correo electrónico
	 * 
	 * @param email correo electrónico.
	 * @return {@link User} si el usuario existe en el sistema, null en caso contrario.
	 */
	public User getByEmail(String email);
}
