package edu.uoc.phonebook.business.controllers;

import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.exceptions.EmailAlreadyExistsException;
import edu.uoc.phonebook.exceptions.InvalidCredentialException;
import edu.uoc.phonebook.exceptions.InvalidPasswordException;
import edu.uoc.phonebook.exceptions.UserAlreadyExistsException;
import edu.uoc.phonebook.integration.controllers.IntegrationUserController;
import edu.uoc.phonebook.presentation.IBusinessUserController;

public class BusinessUserController implements IBusinessUserController {

	private IntegrationUserController integrationUserController = null;
	
	public BusinessUserController() {
		integrationUserController = new IntegrationUserController();
	}
	
	public User getUser(String username, String password) {
		return integrationUserController.get(username, password);
	}
	
	public User getUserByUsername (String username)	{
		return integrationUserController.getByUsername(username);
	}
	
	public boolean createUser (String username, String password, String email)
	throws InvalidPasswordException, UserAlreadyExistsException, EmailAlreadyExistsException
	{
		boolean userCreated = false;
		User user = null;
		
		//Se comprueba que el password siga las reglas de seguridad.
		if (checkValidPassword(password))
		{
			//Se comprueba que no exista un usuario con el mismo username
			if (getUserByUsername(username)!=null)
				throw new UserAlreadyExistsException();
			else
			{
				if (getUserByEmail(email)!=null)
					throw new EmailAlreadyExistsException();
				else
				{
					user = new User();
					user.setUsername(username);
					user.setPassword(password);
					user.setEmail(email);
					userCreated = new IntegrationUserController().create(user);
				}
			}
		}
		else
			throw new InvalidPasswordException();

		return userCreated;
	}
	
	public boolean checkValidPassword (String password) {
		if (password.length()<=5)
			return false;
		return true;
	}
	
	public boolean updatePassword (String username, String oldPassword, String newPassword)
	throws InvalidCredentialException, InvalidPasswordException 
	{
		boolean passwordUpdated = false;
		User user = null;
		//Hay que comprobar que el antiguo password pertenezca al del usuario.
		if ((user = integrationUserController.get(username, oldPassword))==null)
			throw new InvalidCredentialException();
		else
		{
			if (!checkValidPassword(newPassword))
				throw new InvalidPasswordException();
			else
			{
				user.setPassword(newPassword);
				passwordUpdated = new IntegrationUserController().update(user);
			}
		}
		return passwordUpdated;
	}
	
	public User getUserByEmail (String email) {
		return integrationUserController.getByEmail(email);
	}
}