package edu.uoc.phonebook.integration;

import org.apache.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;

public class HibernateSessionFactory
{
	private static SessionFactory sessionFactory = null;

	private static final Logger LOG = Logger.getLogger(HibernateSessionFactory.class);
	
	/**
	 * Método encargado de configurar e inicializar la factoría de sesiones de la aplicación.
	 */
    private static synchronized void initializeSessionFactory()
    {
    	Configuration configuration = null;
    	ServiceRegistry serviceRegistry = null;
		try
    	{
			if (sessionFactory==null)
			{
				configuration = new Configuration().configure("resources/database/hibernate.cfg.xml");
				serviceRegistry = new ServiceRegistryBuilder().applySettings(configuration.getProperties()).buildServiceRegistry();
				sessionFactory = configuration.buildSessionFactory(serviceRegistry);
				LOG.debug("Hibernate se ha configurado correctamente");
			}
    	}
		catch (HibernateException e)
    	{
    		LOG.fatal("Error durante la carga de la configuración de Hibernate", e);
    	}
    	catch (NullPointerException e)
    	{
    		LOG.fatal("Error durante la carga de la configuración de Hibernate", e);
    	}
    }

    /**
     * Método que destruye la factoría de sesiones existente
     */
    public void destroySessionFactory()
    {
    	if (sessionFactory!=null)
    	{
			try
    		{
    			sessionFactory.close();
    		}
    		catch (HibernateException e)
    		{
    			LOG.fatal("Error al destruir la factoría de sesiones de Hibernate");
    		}
    	}
    }

    /**
     * Método que recupera la factoría de sesiones que utilizará la aplicación
     * para realizar las consultas a BD. 
     * 
     * @return objeto {@link SessionFactory} con la factoría de sesiones configurada
     * e inicializada.
     */
    public static SessionFactory getSessionFactory()
    {
    	if (sessionFactory==null)
    		initializeSessionFactory();
   		return sessionFactory;
    }
}
