package edu.uoc.phonebook.integration.controllers;

import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;

import edu.uoc.phonebook.business.IIntegrationContactController;
import edu.uoc.phonebook.business.dataobject.Contact;

public class IntegrationContactController 
extends BaseController 
implements IIntegrationContactController
{

	public boolean create(Contact contact)
	{
		boolean contactCreated = false;
		Session session = null;
		Transaction transaction = null;
		try
		{
			session = getSession();
			transaction = session.beginTransaction();
			session.save(contact);
			transaction.commit();
			contactCreated = true;
		}
		catch (HibernateException e)
		{
			transaction.rollback();
		}
		finally
		{
			releaseSession(session);
		}
		return contactCreated;
	}
	
	public boolean update(Contact contact)
	{
		boolean contactUpdated = false;
		Session session = null;
		Transaction transaction = null;
		try
		{
			session = getSession();
			transaction = session.beginTransaction();
			session.update(contact);
			transaction.commit();
			contactUpdated = true;
		}
		catch (HibernateException e)
		{
			transaction.rollback();
		}
		finally
		{
			releaseSession(session);
		}
		return contactUpdated;
	}
	
	public boolean delete (Integer contactId, Integer userId)
	{
		boolean contactDeleted = false;
		Session session = null;
		Transaction transaction = null;
		Contact contact = null;
		try
		{
			session = getSession();
			transaction = session.beginTransaction();
			contact = loadById(contactId, userId);
			if (contact!=null)
			{
				session.delete(contact);
				transaction.commit();
				contactDeleted = true;
			}
		}
		catch (HibernateException e)
		{
			transaction.rollback();
		}
		finally
		{
			releaseSession(session);
		}
		return contactDeleted;
	}
	
	public Contact loadById (Integer contactId, Integer userId)
	{
		Contact contact = null;
		Session session = null;
    	Transaction transaction = null;
    	Criteria criteria = null;
    	try
    	{
    		session = getSession();
    		transaction = session.beginTransaction();
    		
    		criteria = session.createCriteria(Contact.class);
    		criteria.add(Restrictions.eq("contactId", contactId));
    		criteria.add(Restrictions.eq("userId", userId));
    		
    		contact = (Contact)criteria.uniqueResult();
    		transaction.commit();
    	}
    	catch (HibernateException e)
    	{
    		transaction.rollback();
    	}
    	finally
    	{
    		releaseSession(session);
    	}
		return contact;
	}

	@SuppressWarnings("unchecked")
	public List<Contact> find (String name, String phone, String town, String zipCode, Date startDate, Date endDate, Integer userId)
	{
		List<Contact> contacts = null;
		Session session = null;
    	Transaction transaction = null;
    	Criteria criteria = null;
    	try
    	{
    		session = getSession();
    		transaction = session.beginTransaction();
    		criteria = session.createCriteria(Contact.class);  		
    		
    		if (!StringUtils.isBlank(name))
    			criteria.add(Restrictions.like("name", name, MatchMode.START));
    		
    		if (!StringUtils.isBlank(phone))
    			criteria.add(Restrictions.eq("phone", phone));
    		
    		if (!StringUtils.isBlank(zipCode))
    			criteria.add(Restrictions.eq("zipCode", zipCode));
    		
    		if (!StringUtils.isBlank(town))
    			criteria.add(Restrictions.eq("town", town));
    		
    		if (startDate!=null)
    			criteria.add(Restrictions.ge("birthdate", startDate));
    		
    		if (endDate!=null)
    			criteria.add(Restrictions.le("birthdate", endDate));
    		
    		criteria.add(Restrictions.eq("userId",userId));
    		
    		contacts = (List<Contact>)criteria.list();
    		transaction.commit();
    	}
    	catch (HibernateException e)
    	{
    		transaction.rollback();
    	}
    	finally
    	{
    		releaseSession(session);
    	}
		return contacts;
	}
	
	public Contact getContactByEmail(String email, Integer userId)
	{
		Contact contact = null;
		Session session = null;
    	Transaction transaction = null;
    	Criteria criteria = null;
    	try
    	{
    		session = getSession();
    		transaction = session.beginTransaction();
    		
    		criteria = session.createCriteria(Contact.class);
    		
    		if (userId!=null)
    			criteria.add(Restrictions.eq("userId", userId));
    		
    		if (!StringUtils.isBlank(email))
    			criteria.add(Restrictions.eq("email", email));
    		
    		contact = (Contact)criteria.uniqueResult();
    		
    		transaction.commit();
    	}
    	catch (HibernateException e)
    	{
    		if (transaction!=null)
    			transaction.rollback();
    	}
    	finally
    	{
    		releaseSession(session);
    	}
		return contact;
	}
	
	public Contact getContactByPhone(String phone, Integer userId)
	{
		Contact contact = null;
		Session session = null;
    	Transaction transaction = null;
    	Criteria criteria = null;
    	try
    	{
    		session = getSession();
    		transaction = session.beginTransaction();
    		
    		criteria = session.createCriteria(Contact.class);
    		
    		if (userId!=null)
    			criteria.add(Restrictions.eq("userId", userId));
    		
    		if (!StringUtils.isBlank(phone))
    			criteria.add(Restrictions.eq("phone", phone));
    		
    		contact = (Contact)criteria.uniqueResult();
    		
    		transaction.commit();
    	}
    	catch (HibernateException e)
    	{
    		if (transaction!=null)
    			transaction.rollback();
    	}
    	finally
    	{
    		releaseSession(session);
    	}
		return contact;
	}
}