package edu.uoc.phonebook.integration.controllers;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import edu.uoc.phonebook.business.IIntegrationUserController;
import edu.uoc.phonebook.business.dataobject.User;

public class IntegrationUserController 
extends BaseController 
implements IIntegrationUserController
{

	public boolean create(User user)
	{
		boolean contactCreated = false;
		Session session = null;
		Transaction transaction = null;
		try
		{
			session = getSession();
			transaction = session.beginTransaction();
			session.save(user);
			transaction.commit();
			contactCreated = true;
		}
		catch (HibernateException e)
		{
			if (transaction!=null)
    			transaction.rollback();
		}
		finally
		{
			releaseSession(session);
		}
		return contactCreated;
	}
	
	public boolean update(User user)
	{
		boolean contactUpdated = false;
		Session session = null;
		Transaction transaction = null;
		try
		{
			session = getSession();
			transaction = session.beginTransaction();
			session.update(user);
			transaction.commit();
			contactUpdated = true;
		}
		catch (HibernateException e)
		{
			if (transaction!=null)
    			transaction.rollback();
		}
		finally
		{
			releaseSession(session);
		}
		return contactUpdated;
	}
	
	public User get (String username, String password)
	{
		User user = null;
		Session session = null;
    	Transaction transaction = null;
    	Criteria criteria = null;
    	try
    	{
    		session = getSession();
    		transaction = session.beginTransaction();
    		
    		criteria = session.createCriteria(User.class);
    		criteria.add(Restrictions.eq("username", username));
    		criteria.add(Restrictions.eq("password", password));
    		user = (User)criteria.uniqueResult();
    		
    		transaction.commit();
    	}
    	catch (HibernateException e)
    	{
    		if (transaction!=null)
    			transaction.rollback();
    	}
    	finally
    	{
    		releaseSession(session);
    	}
		return user;
	}
	
	public User getByUsername (String username)
	{
		User user = null;
		Session session = null;
    	Transaction transaction = null;
    	Criteria criteria = null;
    	try
    	{
    		session = getSession();
    		transaction = session.beginTransaction();
    		
    		criteria = session.createCriteria(User.class);
    		criteria.add(Restrictions.eq("username", username));
    		user = (User)criteria.uniqueResult();
    		
    		transaction.commit();
    	}
    	catch (HibernateException e)
    	{
    		if (transaction!=null)
    			transaction.rollback();
    	}
    	finally
    	{
    		releaseSession(session);
    	}
		return user;
	}
	
	public User getByEmail(String email)
	{
		User user = null;
		Session session = null;
    	Transaction transaction = null;
    	Criteria criteria = null;
    	try
    	{
    		session = getSession();
    		transaction = session.beginTransaction();
    		
    		criteria = session.createCriteria(User.class);
    		criteria.add(Restrictions.eq("email", email));
    		user = (User)criteria.uniqueResult();
    		
    		transaction.commit();
    	}
    	catch (HibernateException e)
    	{
    		if (transaction!=null)
    			transaction.rollback();
    	}
    	finally
    	{
    		releaseSession(session);
    	}
		return user;
	}
}