package edu.uoc.phonebook.presentation.actions;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.action.ActionSupport;
import edu.uoc.phonebook.business.dataobject.User;

/**
 * Clase base abstracta que todos los action de la aplicación utilizan para
 * canalizar en un único punto la validación sobre si hay o no sesión activa 
 * antes de llevar a cabo ninguna operación. 
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public abstract class ActionBase extends ActionSupport {

	public static final String LOGIN = "login";
	public static final String ERROR = "error";
	public static final String OK = "ok";
	
	/**
	 * Método que lleva a cabo la comprobación sobre si la acción recibida 
	 * puede o no realizarse en función de si el usuario dispone o no de 
	 * sesion activa.
	 */
	public ActionResponse actionExecute() 
	throws Exception
	{
		User user = null;
		ActionResponse actionResponse = null;
		if (getSession()!=null)
		{
			if ((user = (User) getSessionAttribute("user"))!=null)
				actionResponse = doExecute(user);
			else
				actionResponse = new ActionResponse(LOGIN);
		}
		return actionResponse;	
	}
	
	/**
	 * Método que todos los action de la aplicación implementarán para heredar  
	 * la comprobación sobre la validez de la sesión del usuario en el sistema,
	 * antes de llevar a cabo cualquier acción.
	 */
	public abstract ActionResponse doExecute(User user) throws Exception;
}
