package edu.uoc.phonebook.presentation.actions;

import java.util.List;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessContactController;
import edu.uoc.phonebook.business.dataobject.Contact;
import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.exceptions.InvalidDateException;
import edu.uoc.phonebook.presentation.forms.ContactSearchForm;

/**
 * Acción que lleva a cabo la búsqueda de contactos en el sistema para un usuario dado.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ContactSearch extends ActionBase 
{
	
	private static Logger log = Logger.getLogger(ContactSearch.class);
	
	@Override
	public ActionResponse doExecute(User user)
	throws Exception
	{
		List<Contact> contacts = null;
		ActionResponse actionResponse = null;
		ContactSearchForm contactForm = (ContactSearchForm)getForm();

		try
		{
			//Se solicita al controlador de la capa de negocio que lleve a cabo la búsqueda 
			//de contactos en función de los criterios proporcionados.
			contacts = new BusinessContactController().findContacts(
				contactForm.getName(), 
				contactForm.getPhone(),
				contactForm.getZipCode(),
				contactForm.getTown(),
				contactForm.getStartDate(),
				contactForm.getEndDate(),
				user.getUserId());
			
			log.info("La búsqueda de contactos del usuario " + user.getUsername() + " ha arrojado " + (contacts!=null?contacts.size():0) + " resultados");
			//Se genera una respuesta que contiene los contactos encontrados que se asociarán con el atributo "result" y se mostrarán
			//en la vista identificada en el xml de configuración con el nombre 'ok'.
			actionResponse = new ActionResponse("result",contacts, OK);
		}
		catch (InvalidDateException e)
		{
			/* Se ha producido un error que indica que alguna fecha no es válida.
			 * Caso imposible ya que estos datos han sido validados previamente en el formulario asociado.*/
			addErrorMessage("errors", new Message("contactSearch.criteria.dates.error"));
			log.error("Fechas inválidas en la búsqueda de contactos");
		}
		return actionResponse;
	}
}