 package edu.uoc.phonebook.presentation.actions;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.action.ActionSupport;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessUserController;
import edu.uoc.phonebook.exceptions.EmailAlreadyExistsException;
import edu.uoc.phonebook.exceptions.InvalidPasswordException;
import edu.uoc.phonebook.exceptions.UserAlreadyExistsException;
import edu.uoc.phonebook.presentation.IBusinessUserController;
import edu.uoc.phonebook.presentation.forms.CreateUserForm;

/**
 * Acción que lleva a cabo la creación de un contacto en la agenda de un usuario.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class CreateUser extends ActionSupport 
{
	private static final String ERROR = "error";
	private static Logger log = Logger.getLogger(CreateUser.class);
	
	@Override
	public ActionResponse actionExecute() 
	throws Exception 
	{
		ActionResponse actionResponse = null;
		CreateUserForm createUserForm = (CreateUserForm)getForm();
		IBusinessUserController businessUserController = new BusinessUserController();
		try
		{
			//Se solicita al controlador de la capa de negocio que lleve a cabo el alta de un nuevo usuario en el sistema
			//a partir de los datos recuperados del formulario recibido.
			if (businessUserController.createUser(
					createUserForm.getUsername(), 
					createUserForm.getPassword(), 
					createUserForm.getEmail()))
			{
				log.info("Alta correcta del usuario " + createUserForm.getUsername());
				actionResponse = new ActionResponse("ok");
			}
			else
			{
				/* No se ha podido llevar a cabo el alta del usuario
				 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
				log.error("Ha sido imposible realizar el alta del usuario");
				addErrorMessage("errors", new Message("user.creation.error"));
				actionResponse = new ActionResponse("user", createUserForm, ERROR);
			}
		}
		catch (InvalidPasswordException ipe)
		{
			/* Se ha producido un error que indica que el password no es válido
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("La clave de acceso proporcionada no tiene la longitud mínima de 6 carácteres");
			addErrorMessage("errors", new Message("user.creation.newpassword.invalid"));
			actionResponse = new ActionResponse("user", createUserForm, ERROR);
		}
		catch (UserAlreadyExistsException uae)
		{
			/* Se ha producido un error que indica que ya existía un usuario con el mismo nombre de usuario.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("Ya existe un usuario con el username " + createUserForm.getUsername());
			addErrorMessage("errors", new Message("user.creation.username.exists"));
			actionResponse = new ActionResponse("user", createUserForm, ERROR);
		}
		catch (EmailAlreadyExistsException eae)
		{
			/* Se ha producido un error que indica que ya existía un usuario con el mismo email.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("Ya existe un usuario con el email " + createUserForm.getEmail());
			addErrorMessage("errors", new Message("user.creation.email.exists"));
			actionResponse = new ActionResponse("user", createUserForm, ERROR);
		}
		return actionResponse;
	}
}