package edu.uoc.phonebook.presentation.actions;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessContactController;
import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.presentation.IBusinessContactController;

/**
 * Acción que lleva a cabo la eliminación de un contacto de la agenda de un usuario. 
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class DeleteContact extends ActionBase 
{
	private static Logger log = Logger.getLogger(DeleteContact.class);
	@Override
	public ActionResponse doExecute(User user)
	throws Exception
	{
		ActionResponse actionResponse = null;
		Integer contactId = Integer.valueOf(getRequestParameter("contactId"));
		IBusinessContactController businessContactController = new BusinessContactController();
		
		//Se solicita al controlador de la capa de negocio que lleve a cabo la eliminación de un 
		//contacto del usuario.
		if (businessContactController.deleteContact(contactId, user.getUserId()))
		{
			actionResponse = new ActionResponse(OK);
			log.info("El usuario " + user.getUsername() + " ha eliminado correctamente al contacto con código " + contactId);
		}
		else
		{
			/* No se ha podido llevar a cabo la eliminación del contacto.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("No ha sido posible llevar a cabo la eliminación del contacto con código " + contactId);
			addErrorMessage("errors",new Message("contact.delete.error"));
			actionResponse = new ActionResponse(ERROR);
		}
		return actionResponse;
	}
}