package edu.uoc.phonebook.presentation.actions;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessContactController;
import edu.uoc.phonebook.business.dataobject.Contact;
import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.presentation.IBusinessContactController;

/**
 * Acción que permite consultar los datos de un contacto.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class EditContact extends ActionBase 
{
	private static Logger log = Logger.getLogger(EditContact.class);
	
	@Override
	public ActionResponse doExecute(User user)
	throws Exception
	{
		Integer contactId = null;
		ActionResponse actionResponse = null;
		Contact contact = null;
		IBusinessContactController businessContactController = new BusinessContactController();
		
		//Se recupera de la request el parámetro con nombre 'contactId' que contiene el identificador de contacto
		//que quiere consultarse.
		if (!StringUtils.isBlank(getRequestParameter("contactId")))
		{
			if ((contactId = Integer.valueOf(getRequestParameter("contactId")))!=null)
			{
				/* Se solicita a la capa de negocio que recupere los datos del contacto con id 'contactId' para el usuario 'user.getUserId()'. */
				contact = businessContactController.getContact(contactId, user.getUserId());
				if (contact != null)
				{
					//Se crea una respuesta que envía los datos del contacto al destino con nombre ok.
					actionResponse = new ActionResponse("contact", contact, OK);
					log.info("Se han recuperado los datos del contacto " + contactId + " del usuario " + user.getUsername());
				}
				else
				{
					/* No se ha podido llevar a cabo la edición del contacto.
					 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
					log.error("Imposible recuperar el contacto con id " + contactId);
					addErrorMessage("errors",new Message("contact.edit.error"));
					actionResponse = new ActionResponse(ERROR);
				}
			}	
		}
		else
		{
			/* No se ha podido llevar a cabo la edición del contacto.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("Imposible recuperar un contacto cuyo contactId es null");
			addErrorMessage("errors",new Message("contact.edit.error"));
			actionResponse = new ActionResponse(ERROR);
		}
		
		return actionResponse;
	}
}