package edu.uoc.phonebook.presentation.actions;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.action.ActionSupport;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessUserController;
import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.presentation.IBusinessUserController;
import edu.uoc.phonebook.presentation.forms.LoginForm;

/**
 * Acción encargada de comprobar las credenciales de acceso del usuario en la aplicación.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class Login extends ActionSupport 
{
	private static final String ERROR = "error";
	private static final String OK = "ok";

	private static Logger log = Logger.getLogger(Login.class);
	
	@Override
	public ActionResponse actionExecute() 
	throws Exception 
	{
		User user = null;
		ActionResponse actionResponse = null;
		LoginForm loginForm = (LoginForm)getForm();
		IBusinessUserController businessUserController = new BusinessUserController();
		
		if (loginForm!=null)
		{
			//Se le solicita al controlador de la capa de negocio que recupere los datos del usuario
			//con las credenciales de acceso recibidas en el formulario loginForm.
			if ((user = businessUserController.getUser(loginForm.getUsername(),loginForm.getPassword()))!=null)
			{
				//Se crea un objeto en sesión que contendrá los datos del usuario logado.
				getSession().setAttribute("user", user);
				actionResponse = new ActionResponse(OK);
				log.info("Las credenciales del usuario " + loginForm.getUsername() + " se han validado correctamente");
			}
			else
			{
				/* No se ha encontrado ningún usuario cuyas credenciales de acceso coincidan con las proporcionadas.
				 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
				log.error("Credenciales inválida para el usuario " + loginForm.getUsername());
				addErrorMessage("errors",new Message("login.invalid.credentials"));
				actionResponse = new ActionResponse(ERROR);
			}
		}
		else
		{
			/* No se ha encontrado ningún usuario cuyas credenciales de acceso coincidan con las proporcionadas (inexistentes en este caso).
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("No se han proporcionado credenciales de acceso a la aplicación");
			addErrorMessage("errors",new Message("login.invalid.credentials"));
			actionResponse = new ActionResponse(ERROR);
		}
		return actionResponse;
	}
}