package edu.uoc.phonebook.presentation.actions;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessUserController;
import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.exceptions.InvalidCredentialException;
import edu.uoc.phonebook.exceptions.InvalidPasswordException;
import edu.uoc.phonebook.presentation.IBusinessUserController;
import edu.uoc.phonebook.presentation.forms.UpdatePasswordForm;

/**
 * Acción encargada de llevar a cabo la actualización de las credenciales de acceso de un usuario.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class UpdatePassword extends ActionBase 
{

	private static Logger log = Logger.getLogger(UpdatePassword.class);
	
	/**
	 * Método que lleva a cabo la lógica asociada con una petición que solicita la modificación de la clave de acceso de un usuario.
	 */
	@Override
	public ActionResponse doExecute(User user)
	throws Exception
	{
		ActionResponse actionResponse = null;
		IBusinessUserController businessUserController = new BusinessUserController();
		UpdatePasswordForm updatePasswordForm = (UpdatePasswordForm) getForm();
		try
		{
			//Se pide al controlador de la capa de negocio que lleve a cabo la actualización del password, a partir de los datos
			//recuperados del formulario enviado.
			if (businessUserController.updatePassword(
					user.getUsername(),
					updatePasswordForm.getOldPassword(),
					updatePasswordForm.getNewPassword()))
			{
				user.setPassword(updatePasswordForm.getNewPassword());
				setSessionAttribute("user", user);
				actionResponse = new ActionResponse("success", new Message("user.updatepassword.successful"), OK);
				log.info("El usuario " + user.getUsername() + " ha actualizado su clave de acceso correctamente");
			}
			else
			{	
				/* No se ha podido llevar a cabo la actualización de las credenciales de acceso del usuario.
				 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
				log.error("Ha sido imposible modificar la clave de acceso del usuario " + user.getUsername());
				addErrorMessage("errors",new Message("user.updatepassword.error"));
				actionResponse = new ActionResponse(ERROR);
			}
		}
		catch (InvalidPasswordException ipe)
		{
			/* El nuevo password no cumple las expectativas de seguridad.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("La nueva clave de acceso no tiene la longitud mínima. 6 caracteres");
			addErrorMessage("errors",new Message("user.newpassword.invalid"));
			actionResponse = new ActionResponse(ERROR);
		}
		catch (InvalidCredentialException ic)
		{
			/* El antiguo password no coincide con el password del usuario logado.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("La clave de acceso actual proporcionada no es válida");
			addErrorMessage("errors",new Message("user.oldpassword.invalid"));
			actionResponse = new ActionResponse(ERROR);
		}
		return actionResponse;
	}
}