package edu.uoc.phonebook.presentation.exceptions;

import org.apache.log4j.Logger;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.exceptionHandler.ExceptionHandler;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.helpers.Data;

/**
 * Clase encargada de gestionar las excepciones que se produzcan en la aplicación.
 * El tipo de excepción que manejará esta clase ha sido declarado previamente
 * en el fichero XML de configuración.
 * 
 *<pre>
 *{@code
 *<jmvc:exceptions>
 *	<jmvc:exception jsp="/jsp/error.jsp" type="java.lang.Exception" handler="edu.uoc.phonebook.presentation.exceptions.PhonebookExceptionHandler"/>
 *</jmvc:exceptions>
 *}
 *</pre>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class PhonebookExceptionHandler implements ExceptionHandler
{
	private static final Logger LOG = Logger.getLogger(PhonebookExceptionHandler.class);
	
	@Override
	public Data execute(ContextObject contextObject, Form form, Exception exception) 
	throws Exception 
	{
		//Se deja constancia en el LOG de la aplicación de la excepción sucedida.
		LOG.error("Se ha producido un error no controlado " + exception);
		return new Data("error",exception);
	}
}