package edu.uoc.phonebook.presentation.forms;

import java.text.ParseException;

import org.apache.commons.lang3.StringUtils;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.form.FormErrors;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.utils.Utils;
import edu.uoc.phonebook.utils.Utils.FormatosFecha;

/**
 * 
 * Clase que representa el formulario de datos de un contacto. 
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ContactForm implements Form 
{
	private String name;
	private String surname;
	private String email;
	private String address;
	private String town;
	private String phone;
	private String zipCode;
	private String birthDate;
	private Integer contactId;
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public String getTown() {
		return town;
	}

	public void setTown(String town) {
		this.town = town;
	}

	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public String getZipCode() {
		return zipCode;
	}

	public void setZipCode(String zipCode) {
		this.zipCode = zipCode;
	}

	public String getBirthDate() {
		return birthDate;
	}

	public void setBirthDate(String birthDate) {
		this.birthDate = birthDate;
	}

	public Integer getContactId() {
		return contactId;
	}

	public void setContactId(Integer contactId) {
		this.contactId = contactId;
	}

	/**
	 * Método que comprueba la validez de los campos enviados en el formulario de un contacto.
	 */
	public FormErrors validate(ContextObject contextObject)
	throws Exception 
	{
		FormErrors formErrors = new FormErrors();
		//Se comprueba si el nombre del contacto ha sido informado.
		if (StringUtils.isBlank(name))
			formErrors.addError("name", new Message("contact.name.field.empty"));
		
		//Se comprueba si el teléfono del contacto ha sido informado.
		if (StringUtils.isBlank(phone))
			formErrors.addError("phone", new Message("contact.phone.field.empty"));
		
		//Se comprueba si el correo electrónico del contacto ha sido informado.
		if (!StringUtils.isBlank(email))
		{
			//Se comprueba que el correo electrónico sea válido.
			if (!Utils.isEmailValid(email))
				formErrors.addError("email", new Message("contact.email.field.invalid"));
		}
		//Se comprueba si la fecha de nacimiento ha sido informada.
		if (StringUtils.isNotBlank(birthDate))
		{
			try
			{
				//Se comprueba que la fecha sea válida y tenga el formato correcto.
				Utils.parseDate(birthDate, FormatosFecha.DMY);
			}
			catch (ParseException e)
			{
				formErrors.addError("birthDate", new Message("contact.birthDate.error"));
			}
		}
		return formErrors;
	}
}