package edu.uoc.phonebook.presentation.forms;

import java.text.ParseException;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.form.FormErrors;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.utils.Utils;
import edu.uoc.phonebook.utils.Utils.FormatosFecha;

/**
 * 
 * Clase que representa el formulario de búsqueda de contactos. 
 * 
 * @author Alejandro Marmelo Insúa
 */
public class ContactSearchForm implements Form 
{
	private String name;
	private String town;
	private String phone;
	private String zipCode;
	private String startDate;
	private String endDate;

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getTown() {
		return town;
	}

	public void setTown(String town) {
		this.town = town;
	}

	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	public String getZipCode() {
		return zipCode;
	}

	public void setZipCode(String zipCode) {
		this.zipCode = zipCode;
	}

	public String getStartDate() {
		return startDate;
	}

	public void setStartDate(String startDate) {
		this.startDate = startDate;
	}

	public String getEndDate() {
		return endDate;
	}

	public void setEndDate(String endDate) {
		this.endDate = endDate;
	}

	/**
	 * Método que comprueba la validez de los campos enviados en el formulario de búsqueda de contactos.
	 */
	public FormErrors validate(ContextObject contextObject)
	throws Exception 
	{
		Date sDate = null;
		Date eDate = null;
		FormErrors formErrors = new FormErrors();
		
		//Se comprueba si la fecha de inicio ha sido informada.
		if (!StringUtils.isBlank(startDate))
		{
			try
			{	//Se comprueba que la fecha de inicio del intervalo de búsqueda esté bien formada y sea válida.
				sDate = Utils.parseDate(startDate, FormatosFecha.DMY);
			}
			catch (ParseException pe)
			{
				formErrors.addError("startDate", new Message("contactsearch.startdate.invalid"));
			}
		}
		//Se comprueba si la fecha de fin ha sido informada.
		if (!StringUtils.isBlank(endDate))
		{
			try
			{ //Se comprueba que la fecha de fin del intervalo de búsqueda esté bien formada y sea válida.
				eDate = Utils.parseDate(endDate, FormatosFecha.DMY);
			}
			catch (ParseException pe)
			{
				formErrors.addError("endDate", new Message("contactsearch.enddate.invalid"));
			}
		}
		
		if (eDate!=null && sDate!=null)
		{
			//Se comprueba si el intervalo de fechas indicadas en el formulario es válido.
			if (sDate.after(eDate))
				formErrors.addError("date", new Message("contactsearch.date.interval.error"));
		}
		return formErrors;
	}
}