package edu.uoc.phonebook.presentation.forms;

import org.apache.commons.lang3.StringUtils;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.form.FormErrors;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.utils.Utils;

/**
 * 
 * Clase que representa el formulario de alta de un nuevo usuario. 
 * 
 * @author Alejandro Marmelo Insúa
 */
public class CreateUserForm implements Form 
{
	private String username;
	private String password;
	private String repassword;
	private String email;
	
	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getRepassword() {
		return repassword;
	}

	public void setRepassword(String repassword) {
		this.repassword = repassword;
	}
	
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * Método que comprueba la validez de los campos enviados en el formulario creación de un nuevo usuario.
	 */
	public FormErrors validate(ContextObject contextObject)
	throws Exception 
	{
		FormErrors formErrors = new FormErrors();
		
		//Se comprueba que el username se haya informado.
		if (StringUtils.isBlank(username))
			formErrors.addError("username", new Message("user.username.field.empty"));
		
		//Se comprueba que la clave de acceso se haya informado.
		if (StringUtils.isBlank(password))
			formErrors.addError("password", new Message("user.password.field.empty"));
		
		//Se comprueba que la confirmación de la clave de acceso se haya informado.
		if (StringUtils.isBlank(repassword))
			formErrors.addError("repassword", new Message("user.repassword.field.empty"));
		else
		{
			//Se comprueba que la nueva clave de acceso y su confirmación coincidan exactamente.
			if (!StringUtils.equals(password, repassword))
				formErrors.addError("repassword", new Message("user.passwords.not.match"));
		}
		//Se comprueba que el email se haya informado.
		if (StringUtils.isBlank(email))
			formErrors.addError("email", new Message("user.email.field.empty"));
		else
		{	//Se comprueba que el email sea válido.
		    if (!Utils.isEmailValid(email))
				formErrors.addError("email", new Message("user.email.invalid"));
		}
		return formErrors;
	}
}
