package edu.uoc.phonebook.presentation.forms;

import org.apache.commons.lang3.StringUtils;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.form.FormErrors;
import edu.uoc.framework.utilities.I18N.Message;

/**
 * Clase que representa los datos enviados en el formulario de actualización de la clave de acceso al sistema.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class UpdatePasswordForm implements Form 
{
	private String oldPassword;
	private String newPassword;
	private String newRePassword;

	public String getOldPassword() {
		return oldPassword;
	}

	public void setOldPassword(String oldPassword) {
		this.oldPassword = oldPassword;
	}

	public String getNewPassword() {
		return newPassword;
	}

	public void setNewPassword(String newPassword) {
		this.newPassword = newPassword;
	}

	public String getNewRePassword() {
		return newRePassword;
	}

	public void setNewRePassword(String newRePassword) {
		this.newRePassword = newRePassword;
	}

	/**
	 * Método que comprueba los datos enviados en el formulario de actualización de contraseña. 
	 */
	public FormErrors validate(ContextObject contextObject)
	throws Exception 
	{
		FormErrors formErrors = new FormErrors();
		
		//Se comprueba que el password actual se haya informado.
		if (StringUtils.isBlank(oldPassword))
			formErrors.addError("oldPassword", new Message("user.oldpassword.field.empty"));

		//Se comprueba que el nuevo password se haya informado.
		if (StringUtils.isBlank(newPassword))
			formErrors.addError("newPassword", new Message("user.newpassword.field.empty"));
		
		//Se comprueba que la confirmación del nuevo password se haya informado.
		if (StringUtils.isBlank(newRePassword))
			formErrors.addError("newRePassword", new Message("user.newrepassword.field.empty"));
		else
		{
			//Se comprueba que el nuevo password y su confirmación coincidan exactamente.
			if (!StringUtils.equals(newPassword, newRePassword))
				formErrors.addError("newRePassword", new Message("user.passwords.not.match"));
		}
		return formErrors;
	}
}