package edu.uoc.phonebook.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;

public class Utils 
{
	public static enum FormatosFecha {DMYHMS, DMY, YMD, YMDHMS};

	public static SimpleDateFormat formatterDMY = new SimpleDateFormat ("dd-MM-yyyy");
	private static SimpleDateFormat formatterDMYHMS = new SimpleDateFormat ("dd-MM-yyyy HH:mm:ss");
	private static SimpleDateFormat formatterYMD = new SimpleDateFormat ("yyyy-MM-dd");
	private static SimpleDateFormat formatterYMDHMS = new SimpleDateFormat ("yyyy-MM-dd HH:mm:ss");

	/**
	 * Método que convierte una fecha en formato String que sigue el patrón indicado por 'formatosFecha', en un objeto Date que representa
	 * esa fecha.
	 * 
	 * @param fecha Cadena que contiene la fecha que se desea convertir.
	 * @param formatosFecha Formato que sigue la fecha 
	 * @return Objeto Date que representa la fecha indicada en 'fecha'.
	 * @throws ParseException Excepción que indica que la cadena no ha podido ser parseada.
	 */
	public static Date parseDate(String fecha, FormatosFecha formatosFecha)
	throws ParseException
	{
		Date date = null;

		if(StringUtils.isNotEmpty(fecha))
		{
			switch (formatosFecha)
			{
				case DMY:
						formatterDMY.setLenient(false);
						date = formatterDMY.parse(fecha);
				break;

				case DMYHMS:
						formatterDMYHMS.setLenient(false);
						date = formatterDMYHMS.parse(fecha);
				break;

				case YMD:
						formatterYMD.setLenient(false);
						date = formatterYMD.parse(fecha);
				break;

				case YMDHMS:
						formatterYMDHMS.setLenient(false);
						date = formatterYMDHMS.parse(fecha);
				break;
			}
		}
		return date;
	}
	
	/**
	 * Método que comprueba que un mail dado sea correcto.
	 * 
	 * @param email Cadena que contiene el email a validar.
	 * @return true si el mail está bien formado, false en cualquier otro caso.
	 */
	public static boolean isEmailValid(String email)
	{
		Pattern pattern = Pattern.compile("([_A-Za-z0-9-]+(\\.[_A-Za-z0-9-]+)*@[A-Za-z0-9]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,}))");
	    Matcher m = pattern.matcher(email);
	    return m.matches();
	}
	
}